@@ -1,4 +1,4 @@
-#  Copyright (c) 2015-2020 by Rocky Bernstein
+#  Copyright (c) 2015-2021 by Rocky Bernstein
 #
 #  This program is free software: you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -68,11 +68,9 @@ from __future__ import print_function
 import re
 
 import decompyle3.parsers.main as python_parser
-from xdis import iscode, sysinfo2float
 from decompyle3.semantics import pysource
 from decompyle3.scanner import Token, Code, get_scanner
 from decompyle3.semantics.check_ast import checker
-from decompyle3 import IS_PYPY
 
 from decompyle3.show import maybe_show_asm, maybe_show_tree
 
@@ -90,9 +88,15 @@ from decompyle3.semantics.consts import (
     PASS,
 )
 
+from decompyle3.semantics.customize import customize_for_version
+from decompyle3.semantics.make_function36 import make_function36
+
 from spark_parser import DEFAULT_DEBUG as PARSER_DEFAULT_DEBUG
 from spark_parser.ast import GenericASTTraversalPruningException
 
+from xdis import iscode
+from xdis.version_info import IS_PYPY, PYTHON_VERSION_TRIPLE
+
 from collections import namedtuple
 
 NodeInfo = namedtuple("NodeInfo", "node start finish")
@@ -185,10 +189,20 @@ class FragmentsWalker(pysource.SourceWalker, object):
         self.hide_internal = False
         self.offsets = {}
         self.last_finish = -1
+        self.debug_parser = dict(debug_parser)
+
+        # FIXME: have p.insts update in a better way
+        # modularity is broken here
+        self.p.insts = scanner.insts
+        self.p.opc = scanner.opc
+        self.p.offset2inst_index = scanner.offset2inst_index
 
         # FIXME: is there a better way?
         global MAP_DIRECT_FRAGMENT
         MAP_DIRECT_FRAGMENT = (dict(TABLE_DIRECT, **TABLE_DIRECT_FRAGMENT),)
+        self.version = version
+        self.is_pypy = is_pypy
+        customize_for_version(self, is_pypy, version)
         return
 
     f = property(
@@ -316,9 +330,10 @@ class FragmentsWalker(pysource.SourceWalker, object):
 
     def n_return(self, node):
         start = len(self.f.getvalue()) + len(self.indent)
-        if (
-                self.params["is_lambda"]
-                or node[0] in ("pop_return", "popb_return", "pop_ex_return")
+        if self.params["is_lambda"] or node[0] in (
+            "pop_return",
+            "popb_return",
+            "pop_ex_return",
         ):
             self.preorder(node[0])
             if hasattr(node[-1], "offset"):
@@ -585,7 +600,7 @@ class FragmentsWalker(pysource.SourceWalker, object):
     def n_mkfunc(self, node):
         start = len(self.f.getvalue())
 
-        if self.version >= 3.3 or node[-2] == "kwargs":
+        if self.version >= (3, 3) or node[-2] == "kwargs":
             # LOAD_CONST code object ..
             # LOAD_CONST        'x0'  if >= 3.3
             # MAKE_FUNCTION ..
@@ -602,7 +617,7 @@ class FragmentsWalker(pysource.SourceWalker, object):
 
         self.indent_more()
         start = len(self.f.getvalue())
-        self.make_function36(node, is_lambda=False, code_node=code_node)
+        make_function36(self, node, is_lambda=False, code_node=code_node)
 
         self.set_pos_info(node, start, len(self.f.getvalue()))
 
@@ -618,24 +633,18 @@ class FragmentsWalker(pysource.SourceWalker, object):
         self.prec = 27
 
         # FIXME: clean this up
-        if self.version > 3.0 and node == "dict_comp":
+        if node == "dict_comp":
             cn = node[1]
-        elif self.version > 3.0 and node == "generator_exp":
+        elif node in ("generator_exp", "generator_exp_async"):
             if node[0] == "load_genexpr":
                 load_genexpr = node[0]
             elif node[1] == "load_genexpr":
                 load_genexpr = node[1]
             cn = load_genexpr[0]
-        elif hasattr(node[code_index], "attr"):
-            # Python 2.5+ (and earlier?) does this
-            cn = node[code_index]
         else:
             if len(node[1]) > 1 and hasattr(node[1][1], "attr"):
                 # Python 3.3+ does this
                 cn = node[1][1]
-            elif hasattr(node[1][0], "attr"):
-                # Python 3.2 does this
-                cn = node[1][0]
             else:
                 assert False, "Can't find code for comprehension"
 
@@ -644,32 +653,52 @@ class FragmentsWalker(pysource.SourceWalker, object):
         code = Code(cn.attr, self.scanner, self.currentclass)
         ast = self.build_ast(code._tokens, code._customize, code)
         self.customize(code._customize)
-        ast = ast[0][0][0]
+
+        # Remove single reductions as in ("stmts", "sstmt"):
+        while len(ast) == 1:
+            ast = ast[0]
 
         n = ast[iter_index]
-        assert n == "comp_iter"
-        # find innermost node
-        while n == "comp_iter":
+
+        assert n == "comp_iter", n.kind
+        # Find the comprehension body. It is the inner-most
+        # node that is not list_.. .
+        while n == "comp_iter":  # list_iter
             n = n[0]  # recurse one step
             if n == "comp_for":
-                n = n[3]
+                if n[0] == "SETUP_LOOP":
+                    n = n[4]
+                else:
+                    n = n[3]
             elif n == "comp_if":
+                n = n[1]
+            elif n == "comp_if_not":
                 n = n[2]
-            elif n == "comp_ifnot":
-                n = n[2]
+
         assert n == "comp_body", ast
 
         self.preorder(n[0])
+        if node == "generator_exp_async":
+            self.write(" async")
+            iter_var_index = iter_index - 2
+        else:
+            iter_var_index = iter_index - 1
         self.write(" for ")
         start = len(self.f.getvalue())
-        store = ast[iter_index - 1]
+        store = ast[iter_var_index]
         self.preorder(store)
         self.set_pos_info(ast[iter_index - 1], start, len(self.f.getvalue()))
         self.write(" in ")
         start = len(self.f.getvalue())
-        node[-3].parent = node
-        self.preorder(node[-3])
-        self.set_pos_info(node[-3], start, len(self.f.getvalue()))
+
+        if node[2] == "expr":
+            iter_expr = node[2]
+        else:
+            iter_expr = node[-3]
+        assert iter_expr == "expr"
+        iter_expr.parent = node
+        self.preorder(iter_expr)
+        self.set_pos_info(iter_expr, start, len(self.f.getvalue()))
         start = len(self.f.getvalue())
         self.preorder(ast[iter_index])
         self.set_pos_info(ast[iter_index], start, len(self.f.getvalue()))
@@ -751,7 +780,7 @@ class FragmentsWalker(pysource.SourceWalker, object):
         self.name = code_name
 
         # Issue created with later Python code generation is that there
-        # is a lamda set up with a dummy argument name that is then called
+        # is a lambda set up with a dummy argument name that is then called
         # So we can't just translate that as is but need to replace the
         # dummy name. Below we are picking out the variable name as seen
         # in the code. And trying to generate code for the other parts
@@ -865,8 +894,8 @@ class FragmentsWalker(pysource.SourceWalker, object):
     def n_generator_exp(self, node):
         start = len(self.f.getvalue())
         self.write("(")
-        code_index = -6 if self.version > 3.2 else -5
-        self.comprehension_walk(node, iter_index=3, code_index=code_index)
+        code_index = -6
+        self.comprehension_walk(node, iter_index=4, code_index=code_index)
         self.write(")")
         self.set_pos_info(node, start, len(self.f.getvalue()))
         self.prune()
@@ -992,7 +1021,7 @@ class FragmentsWalker(pysource.SourceWalker, object):
         # class definition ('class X(A,B,C):')
         cclass = self.currentclass
 
-        if self.version > 3.0:
+        if self.version >= (3, 1):
             if node == "classdefdeco2":
                 currentclass = node[1][2].pattr
                 buildclass = node
@@ -1056,7 +1085,7 @@ class FragmentsWalker(pysource.SourceWalker, object):
         start = len(self.f.getvalue())
         self.write(self.indent, "class ", self.currentclass)
 
-        if self.version > 3.0:
+        if self.version >= (3, 1):
             self.print_super_classes3(subclass_info)
         else:
             self.print_super_classes(build_list)
@@ -1094,8 +1123,15 @@ class FragmentsWalker(pysource.SourceWalker, object):
         self.name = old_name
         self.return_none = rn
 
-    def build_ast(self, tokens, customize, code, is_lambda=False,
-                  noneInNames=False, isTopLevel=False):
+    def build_ast(
+        self,
+        tokens,
+        customize,
+        code,
+        is_lambda=False,
+        noneInNames=False,
+        isTopLevel=False,
+    ):
 
         # FIXME: DRY with pysource.py
 
@@ -1114,10 +1150,11 @@ class FragmentsWalker(pysource.SourceWalker, object):
                 p_insts = self.p.insts
                 self.p.insts = self.scanner.insts
                 self.p.offset2inst_index = self.scanner.offset2inst_index
+                self.p.opc = self.scanner.opc
                 ast = python_parser.parse(self.p, tokens, customize, is_lambda)
                 self.p.insts = p_insts
             except (python_parser.ParserError, AssertionError) as e:
-                raise ParserError(e, tokens)
+                raise ParserError(e, tokens, self.debug_parser.get("reduce", False))
 
             ## FIXME: So as not to remove tokens with offsets,
             ## remove this phase until we have a chance to go over,
@@ -1127,7 +1164,6 @@ class FragmentsWalker(pysource.SourceWalker, object):
             # del ast # Save memory
             # return transform_ast
 
-
         # The bytecode for the end of the main routine has a
         # "return None". However you can't issue a "return" statement in
         # main. In the other build_ast routine we eliminate the
@@ -1158,10 +1194,10 @@ class FragmentsWalker(pysource.SourceWalker, object):
             # modularity is broken here
             p_insts = self.p.insts
             self.p.insts = self.scanner.insts
-            ast = python_parser.parse(self.p, tokens, customize)
+            ast = python_parser.parse(self.p, tokens, customize, is_lambda=is_lambda)
             self.p.insts = p_insts
         except (python_parser.ParserError, AssertionError) as e:
-            raise ParserError(e, tokens)
+            raise ParserError(e, tokens, self.debug_parser.get("reduce", False))
 
         maybe_show_tree(self, ast)
 
@@ -1450,7 +1486,7 @@ class FragmentsWalker(pysource.SourceWalker, object):
         self.write("{")
         self.set_pos_info(node[0], start, start + 1)
 
-        if self.version >= 3.0 and not self.is_pypy:
+        if self.version >= (3, 0) and not self.is_pypy:
             if node[0].kind.startswith("kvlist"):
                 # Python 3.5+ style key/value list in dict
                 kv_node = node[0]
@@ -1622,6 +1658,7 @@ class FragmentsWalker(pysource.SourceWalker, object):
         fmt = entry[0]
         arg = 1
         i = 0
+
         lastC = -1
         recurse_node = False
 
@@ -1666,17 +1703,21 @@ class FragmentsWalker(pysource.SourceWalker, object):
 
                 index = entry[arg]
                 if isinstance(index, tuple):
-                    assert node[index[0]] == index[1], (
-                        "at %s[%d], expected %s node; got %s"
-                        % (node.kind, arg, node[index[0]].kind, index[1])
-                    )
+                    if isinstance(index[1], str):
+                        assert node[index[0]] == index[1], (
+                            "at %s[%d], expected %s node; got %s"
+                            % (node.kind, arg, node[index[0]].kind, index[1])
+                        )
+                    else:
+                        assert node[index[0]] in index[1], (
+                            "at %s[%d], expected to be in '%s' node; got '%s'"
+                            % (node.kind, arg, index[1], node[index[0]].kind)
+                        )
+
                     index = index[0]
-                assert isinstance(
-                    index, int
-                ), "at %s[%d], %s should be int or tuple" % (
-                    node.kind,
-                    arg,
-                    type(index),
+                assert isinstance(index, int), (
+                    "at %s[%d], %s should be int or tuple"
+                    % (node.kind, arg, type(index),)
                 )
                 self.preorder(node[index])
 
@@ -1862,10 +1903,11 @@ def code_deparse(
     assert iscode(co)
 
     if version is None:
-        version = sysinfo2float()
+        version = PYTHON_VERSION_TRIPLE
     if is_pypy is None:
         is_pypy = IS_PYPY
 
+    # store final output stream for case of error
     scanner = get_scanner(version, is_pypy=is_pypy)
 
     show_asm = debug_opts.get("asm", None)
@@ -1892,14 +1934,17 @@ def code_deparse(
         is_pypy=is_pypy,
     )
 
-    deparsed.ast = deparsed.build_ast(tokens, customize)
+    isTopLevel = co.co_name == "<module>"
+    deparsed.ast = deparsed.build_ast(tokens, customize, co, isTopLevel=isTopLevel)
 
     assert deparsed.ast == "stmts", "Should have parsed grammar start"
 
-    del tokens  # save memory
+    # save memory
+    del tokens
 
     # convert leading '__doc__ = "..." into doc string
     assert deparsed.ast == "stmts"
+
     (deparsed.mod_globs, nonlocals) = pysource.find_globals_and_nonlocals(
         deparsed.ast, set(), set(), co, version
     )
